@@ -19,24 +19,34 @@ from .backend import json
 from .compat import numeric_types, string_types, PY3, PY2
 
 
-def encode(value,
-           unpicklable=True,
-           make_refs=True,
-           keys=False,
-           max_depth=None,
-           reset=True,
-           backend=None,
-           warn=False,
-           context=None,
-           max_iter=None,
-           use_decimal=False,
-           numeric_keys=False,
-           use_base85=False):
+def encode(
+    value,
+    unpicklable=True,
+    make_refs=True,
+    keys=False,
+    max_depth=None,
+    reset=True,
+    backend=None,
+    warn=False,
+    context=None,
+    max_iter=None,
+    use_decimal=False,
+    numeric_keys=False,
+    use_base85=False,
+    fail_safe=None,
+    indent=None,
+    separators=None,
+):
     """Return a JSON formatted representation of value, a Python object.
 
-    :param unpicklable: If set to False then the output will not contain the
+    :param unpicklable: If set to ``False`` then the output will not contain the
         information necessary to turn the JSON data back into Python objects,
-        but a simpler JSON stream is produced.
+        but a simpler JSON stream is produced. It's recommended to set this
+        parameter to ``False`` when your code does not rely on two objects
+        having the same ``id()`` value, and when it is sufficient for those two
+        objects to be equal by ``==``, such as when serializing sklearn
+        instances. If you experience (de)serialization being incorrect when you
+        use numpy, pandas, or sklearn handlers, this should be set to ``False``.
     :param max_depth: If set to a non-negative integer then jsonpickle will
         not recurse deeper than 'max_depth' steps into the object.  Anything
         deeper than 'max_depth' is represented using a Python repr() of the
@@ -48,6 +58,14 @@ def encode(value,
         by calling repr() instead of recursing when make_refs is set False.
     :param keys: If set to True then jsonpickle will encode non-string
         dictionary keys instead of coercing them into strings via `repr()`.
+        This is typically what you want if you need to support Integer or
+        objects as dictionary keys.
+    :param numeric_keys: Only use this option if the backend supports integer
+        dict keys natively.  This flag tells jsonpickle to leave numeric keys
+        as-is rather than conforming them to json-friendly strings.
+        Using ``keys=True`` is the typical solution for integer keys, so only
+        use this if you have a specific use case where you want to allow the
+        backend to handle serialization of numeric dict keys.
     :param warn: If set to True then jsonpickle will warn when it
         returns None for an object which it cannot pickle
         (e.g. file descriptors).
@@ -66,6 +84,27 @@ def encode(value,
 
         NOTE: A side-effect of the above settings is that float values will be
         converted to Decimal when converting to json.
+    :param use_base85:
+        If possible, use base85 to encode binary data. Base85 bloats binary data
+        by 1/4 as opposed to base64, which expands it by 1/3. This argument is
+        ignored on Python 2 because it doesn't support it.
+    :param fail_safe: If set to a function exceptions are ignored when pickling
+        and if a exception happens the function is called and the return value
+        is used as the value for the object that caused the error
+    :param indent: When `indent` is a non-negative integer, then JSON array
+        elements and object members will be pretty-printed with that indent
+        level.  An indent level of 0 will only insert newlines. ``None`` is
+        the most compact representation.  Since the default item separator is
+        ``(', ', ': ')``,  the output might include trailing whitespace when
+        ``indent`` is specified.  You can use ``separators=(',', ': ')`` to
+        avoid this.  This value is passed directly to the active JSON backend
+        library and not used by jsonpickle directly.
+    :param separators:
+        If ``separators`` is an ``(item_separator, dict_separator)`` tuple
+        then it will be used instead of the default ``(', ', ': ')``
+        separators.  ``(',', ':')`` is the most compact JSON representation.
+        This value is passed directly to the active JSON backend library and
+        not used by jsonpickle directly.
 
     >>> encode('my string') == '"my string"'
     True
@@ -76,39 +115,41 @@ def encode(value,
     >>> encode({'foo': [1, 2, [3, 4]]}, max_depth=1)
     '{"foo": "[1, 2, [3, 4]]"}'
 
-    :param use_base85:
-        If possible, use base85 to encode binary data. Base85 bloats binary data
-        by 1/4 as opposed to base64, which expands it by 1/3. This argument is
-        ignored on Python 2 because it doesn't support it.
     """
     backend = backend or json
     context = context or Pickler(
-            unpicklable=unpicklable,
-            make_refs=make_refs,
-            keys=keys,
-            backend=backend,
-            max_depth=max_depth,
-            warn=warn,
-            max_iter=max_iter,
-            numeric_keys=numeric_keys,
-            use_decimal=use_decimal,
-            use_base85=use_base85)
-    return backend.encode(context.flatten(value, reset=reset))
+        unpicklable=unpicklable,
+        make_refs=make_refs,
+        keys=keys,
+        backend=backend,
+        max_depth=max_depth,
+        warn=warn,
+        max_iter=max_iter,
+        numeric_keys=numeric_keys,
+        use_decimal=use_decimal,
+        use_base85=use_base85,
+        fail_safe=fail_safe,
+    )
+    return backend.encode(
+        context.flatten(value, reset=reset), indent=indent, separators=separators
+    )
 
 
 class Pickler(object):
-
-    def __init__(self,
-                 unpicklable=True,
-                 make_refs=True,
-                 max_depth=None,
-                 backend=None,
-                 keys=False,
-                 warn=False,
-                 max_iter=None,
-                 numeric_keys=False,
-                 use_decimal=False,
-                 use_base85=False):
+    def __init__(
+        self,
+        unpicklable=True,
+        make_refs=True,
+        max_depth=None,
+        backend=None,
+        keys=False,
+        warn=False,
+        max_iter=None,
+        numeric_keys=False,
+        use_decimal=False,
+        use_base85=False,
+        fail_safe=None,
+    ):
         self.unpicklable = unpicklable
         self.make_refs = make_refs
         self.backend = backend or json
@@ -128,6 +169,8 @@ class Pickler(object):
         self._max_iter = max_iter
         # Whether to allow decimals to pass-through
         self._use_decimal = use_decimal
+        # A cache of objects that have already been flattened.
+        self._flattened = {}
 
         if self.use_base85:
             self._bytes_tag = tags.B85
@@ -136,14 +179,17 @@ class Pickler(object):
             self._bytes_tag = tags.B64
             self._bytes_encoder = util.b64encode
 
+        # ignore exceptions
+        self.fail_safe = fail_safe
+
     def reset(self):
         self._objs = {}
         self._depth = -1
         self._seen = []
+        self._flattened = {}
 
     def _push(self):
-        """Steps down one level in the namespace.
-        """
+        """Steps down one level in the namespace."""
         self._depth += 1
 
     def _pop(self, value):
@@ -214,78 +260,96 @@ class Pickler(object):
         return self._flatten(obj)
 
     def _flatten(self, obj):
+        if self.unpicklable and self.make_refs:
+            result = self._flatten_impl(obj)
+        else:
+            try:
+                result = self._flattened[id(obj)]
+            except KeyError:
+                result = self._flattened[id(obj)] = self._flatten_impl(obj)
+        return result
+
+    def _flatten_impl(self, obj):
+        #########################################
+        # if obj is nonrecursive return immediately
+        # for performance reasons we don't want to do recursive checks
+        if PY2 and isinstance(obj, types.FileType):
+            return self._flatten_file(obj)
+
+        if type(obj) is bytes:
+            return self._flatten_bytestring(obj)
+
+        # Decimal is a primitive when use_decimal is True
+        if type(obj) in util.PRIMITIVES or (
+            self._use_decimal and isinstance(obj, decimal.Decimal)
+        ):
+            return obj
+        #########################################
+
         self._push()
         return self._pop(self._flatten_obj(obj))
 
+    def _max_reached(self):
+        return self._depth == self._max_depth
+
     def _flatten_obj(self, obj):
         self._seen.append(obj)
-        max_reached = self._depth == self._max_depth
 
-        in_cycle = (
-            max_reached or (
-                not self.make_refs
-                and id(obj) in self._objs
-            )) and not util.is_primitive(obj)
-        if in_cycle:
-            # break the cycle
-            flatten_func = repr
-        else:
-            flatten_func = self._get_flattener(obj)
+        max_reached = self._max_reached()
 
-        if flatten_func is None:
-            self._pickle_warning(obj)
-            return None
+        try:
 
-        return flatten_func(obj)
+            in_cycle = _in_cycle(obj, self._objs, max_reached, self.make_refs)
+            if in_cycle:
+                # break the cycle
+                flatten_func = repr
+            else:
+                flatten_func = self._get_flattener(obj)
+
+            if flatten_func is None:
+                self._pickle_warning(obj)
+                return None
+
+            return flatten_func(obj)
+
+        except (KeyboardInterrupt, SystemExit) as e:
+            raise e
+        except Exception as e:
+            if self.fail_safe is None:
+                raise e
+            else:
+                return self.fail_safe(e)
 
     def _list_recurse(self, obj):
         return [self._flatten(v) for v in obj]
 
     def _get_flattener(self, obj):
-
-        if PY2 and isinstance(obj, types.FileType):
-            return self._flatten_file
-
-        if util.is_bytes(obj):
-            return self._flatten_bytestring
-
-        if util.is_primitive(obj):
-            return lambda obj: obj
-
-        # Decimal is a primitive when use_decimal is True
-        if self._use_decimal and isinstance(obj, decimal.Decimal):
-            return lambda obj: obj
-
-        list_recurse = self._list_recurse
-
-        if util.is_list(obj):
+        if type(obj) in (list, dict):
             if self._mkref(obj):
-                return list_recurse
+                return (
+                    self._list_recurse if type(obj) is list else self._flatten_dict_obj
+                )
             else:
                 self._push()
                 return self._getref
 
         # We handle tuples and sets by encoding them in a "(tuple|set)dict"
-        if util.is_tuple(obj):
+        elif type(obj) in (tuple, set):
             if not self.unpicklable:
-                return list_recurse
-            return lambda obj: {tags.TUPLE: [self._flatten(v) for v in obj]}
+                return self._list_recurse
+            return lambda obj: {
+                tags.TUPLE
+                if type(obj) is tuple
+                else tags.SET: [self._flatten(v) for v in obj]
+            }
 
-        if util.is_set(obj):
-            if not self.unpicklable:
-                return list_recurse
-            return lambda obj: {tags.SET: [self._flatten(v) for v in obj]}
-
-        if util.is_dictionary(obj):
-            return self._flatten_dict_obj
-
-        if util.is_type(obj):
-            return _mktyperef
-
-        if util.is_object(obj):
+        elif util.is_object(obj):
             return self._ref_obj_instance
 
-        if util.is_module_function(obj):
+        elif util.is_type(obj):
+            return _mktyperef
+
+        elif util.is_module_function(obj):
             return self._flatten_function
 
         # instance methods, lambdas, old style classes...
@@ -293,16 +357,25 @@ class Pickler(object):
         return None
 
     def _ref_obj_instance(self, obj):
-        """Reference an existing object or flatten if new
-        """
-        if self._mkref(obj):
-            # We've never seen this object so return its
-            # json representation.
+        """Reference an existing object or flatten if new"""
+        if self.unpicklable:
+            if self._mkref(obj):
+                # We've never seen this object so return its
+                # json representation.
+                return self._flatten_obj_instance(obj)
+            # We've seen this object before so place an object
+            # reference tag in the data. This avoids infinite recursion
+            # when processing cyclical objects.
+            return self._getref(obj)
+        else:
+            max_reached = self._max_reached()
+            in_cycle = _in_cycle(obj, self._objs, max_reached, False)
+            if in_cycle:
+                # A circular becomes None.
+                return None
+
+            self._mkref(obj)
             return self._flatten_obj_instance(obj)
-        # We've seen this object before so place an object
-        # reference tag in the data. This avoids infinite recursion
-        # when processing cyclical objects.
-        return self._getref(obj)
 
     def _flatten_file(self, obj):
         """
@@ -320,8 +393,7 @@ class Pickler(object):
         return {self._bytes_tag: self._bytes_encoder(obj)}
 
     def _flatten_obj_instance(self, obj):
-        """Recursively flatten an instance and return a json-friendly dict
-        """
+        """Recursively flatten an instance and return a json-friendly dict"""
         data = {}
         has_class = hasattr(obj, '__class__')
         has_dict = hasattr(obj, '__dict__')
@@ -401,9 +473,12 @@ class Pickler(object):
                 f, args, state, listitems, dictitems = rv_as_list
 
                 # check that getstate/setstate is sane
-                if not (state and hasattr(obj, '__getstate__')
-                        and not hasattr(obj, '__setstate__')
-                        and not isinstance(obj, dict)):
+                if not (
+                    state
+                    and hasattr(obj, '__getstate__')
+                    and not hasattr(obj, '__setstate__')
+                    and not isinstance(obj, dict)
+                ):
                     # turn iterators to iterables for convenient serialization
                     if rv_as_list[3]:
                         rv_as_list[3] = tuple(rv_as_list[3])
@@ -415,7 +490,7 @@ class Pickler(object):
                     last_index = len(reduce_args) - 1
                     while last_index >= 2 and reduce_args[last_index] is None:
                         last_index -= 1
-                    data[tags.REDUCE] = reduce_args[:last_index+1]
+                    data[tags.REDUCE] = reduce_args[: last_index + 1]
 
                     return data
 
@@ -424,8 +499,7 @@ class Pickler(object):
                 data[tags.OBJECT] = class_name
 
             if has_getnewargs_ex:
-                data[tags.NEWARGSEX] = list(
-                    map(self._flatten, obj.__getnewargs_ex__()))
+                data[tags.NEWARGSEX] = list(map(self._flatten, obj.__getnewargs_ex__()))
 
             if has_getnewargs and not has_getnewargs_ex:
                 data[tags.NEWARGS] = self._flatten(obj.__getnewargs__())
@@ -460,8 +534,7 @@ class Pickler(object):
 
         if util.is_iterator(obj):
             # force list in python 3
-            data[tags.ITERATOR] = list(
-                map(self._flatten, islice(obj, self._max_iter)))
+            data[tags.ITERATOR] = list(map(self._flatten, islice(obj, self._max_iter)))
             return data
 
         if has_dict:
@@ -493,14 +566,27 @@ class Pickler(object):
         return data
 
     def _flatten_dict_obj(self, obj, data=None):
-        """Recursively call flatten() and return json-friendly dict
-        """
+        """Recursively call flatten() and return json-friendly dict"""
         if data is None:
             data = obj.__class__()
 
-        flatten = self._flatten_key_value_pair
-        for k, v in sorted(obj.items(), key=util.itemgetter):
-            flatten(k, v, data)
+        # If we allow non-string keys then we have to do a two-phase
+        # encoding to ensure that the reference IDs are deterministic.
+        if self.keys:
+            # Phase 1: serialize regular objects, ignore fancy keys.
+            flatten = self._flatten_string_key_value_pair
+            for k, v in util.items(obj):
+                flatten(k, v, data)
+
+            # Phase 2: serialize non-string keys.
+            flatten = self._flatten_non_string_key_value_pair
+            for k, v in util.items(obj):
+                flatten(k, v, data)
+        else:
+            # If we have string keys only then we only need a single pass.
+            flatten = self._flatten_key_value_pair
+            for k, v in util.items(obj):
+                flatten(k, v, data)
 
         # the collections.defaultdict protocol
         if hasattr(obj, 'default_factory') and callable(obj.default_factory):
@@ -514,8 +600,8 @@ class Pickler(object):
                 if self._mkref(factory):
                     # We've never seen this object before so pickle it in-place.
                     # Create an instance from the factory and assume that the
-                    # resulting instance is a suitable examplar.
-                    value = self._flatten(handlers.CloneFactory(factory()))
+                    # resulting instance is a suitable exemplar.
+                    value = self._flatten_obj_instance(handlers.CloneFactory(factory()))
                 else:
                     # We've seen this object before.
                     # Break the cycle by emitting a reference.
@@ -544,24 +630,56 @@ class Pickler(object):
         return ok
 
     def _flatten_newstyle_with_slots(self, obj, data):
-        """Return a json-friendly dict for new-style objects with __slots__.
-        """
-        allslots = [_wrap_string_slot(getattr(cls, '__slots__', tuple()))
-                    for cls in obj.__class__.mro()]
+        """Return a json-friendly dict for new-style objects with __slots__."""
+        allslots = [
+            _wrap_string_slot(getattr(cls, '__slots__', tuple()))
+            for cls in obj.__class__.mro()
+        ]
 
         if not self._flatten_obj_attrs(obj, chain(*allslots), data):
-            attrs = [x for x in dir(obj)
-                     if not x.startswith('__') and not x.endswith('__')]
+            attrs = [
+                x for x in dir(obj) if not x.startswith('__') and not x.endswith('__')
+            ]
             self._flatten_obj_attrs(obj, attrs, data)
 
         return data
 
     def _flatten_key_value_pair(self, k, v, data):
         """Flatten a key/value pair into the passed-in dictionary."""
+        if not util.is_picklable(k, v):
+            return data
+
+        if k is None:
+            k = 'null'  # for compatibility with common json encoders
+
+        if self.numeric_keys and isinstance(k, numeric_types):
+            pass
+        elif not isinstance(k, string_types):
+            try:
+                k = repr(k)
+            except Exception:
+                k = compat.ustr(k)
+
+        data[k] = self._flatten(v)
+        return data
+
+    def _flatten_non_string_key_value_pair(self, k, v, data):
+        """Flatten only non-string key/value pairs"""
+        if not util.is_picklable(k, v):
+            return data
+        if self.keys and not isinstance(k, string_types):
+            k = self._escape_key(k)
+            data[k] = self._flatten(v)
+        return data
+
+    def _flatten_string_key_value_pair(self, k, v, data):
+        """Flatten string key/value pairs only."""
         if not util.is_picklable(k, v):
             return data
         if self.keys:
-            if not isinstance(k, string_types) or k.startswith(tags.JSON_KEY):
+            if not isinstance(k, string_types):
+                return data
+            elif k.startswith(tags.JSON_KEY):
                 k = self._escape_key(k)
         else:
             if k is None:
@@ -590,10 +708,14 @@ class Pickler(object):
         return data
 
     def _escape_key(self, k):
-        return tags.JSON_KEY + encode(k,
-                                      reset=False, keys=True,
-                                      context=self, backend=self.backend,
-                                      make_refs=self.make_refs)
+        return tags.JSON_KEY + encode(
+            k,
+            reset=False,
+            keys=True,
+            context=self,
+            backend=self.backend,
+            make_refs=self.make_refs,
+        )
 
     def _getstate(self, obj, data):
         state = self._flatten(obj)
@@ -609,6 +731,15 @@ class Pickler(object):
             warnings.warn(msg)
 
 
+def _in_cycle(obj, objs, max_reached, make_refs):
+    """Detect cyclic structures that would lead to infinite recursion"""
+    return (
+        (max_reached or (not make_refs and id(obj) in objs))
+        and not util.is_primitive(obj)
+        and not util.is_enum(obj)
+    )
+
+
 def _mktyperef(obj):
     """Return a typeref dictionary
 
@@ -620,8 +751,7 @@ def _mktyperef(obj):
 
 
 def _wrap_string_slot(string):
-    """Converts __slots__ = 'a' into __slots__ = ('a',)
-    """
+    """Converts __slots__ = 'a' into __slots__ = ('a',)"""
     if isinstance(string, string_types):
         return (string,)
     return string
