@@ -16,8 +16,16 @@ from .compat import numeric_types
 from .backend import json
 
 
-def decode(string, backend=None, context=None, keys=False, reset=True,
-           safe=False, classes=None):
+def decode(
+    string,
+    backend=None,
+    context=None,
+    keys=False,
+    reset=True,
+    safe=False,
+    classes=None,
+    v1_decode=False,
+):
     """Convert a JSON string into a Python object.
 
     The keyword argument 'keys' defaults to False.
@@ -30,13 +38,29 @@ def decode(string, backend=None, context=None, keys=False, reset=True,
     can be used to give jsonpickle access to local classes that are not
     available through the global module import scope.
 
+    The keyword argument 'safe' defaults to False.
+    If set to True, eval() is avoided, but backwards-compatible
+    (pre-0.7.0) deserialization of repr-serialized objects is disabled.
+
+    The keyword argument 'backend' defaults to None.
+    If set to an instance of jsonpickle.backend.JSONBackend, jsonpickle
+    will use that backend for deserialization.
+
+    The keyword argument 'v1_decode' defaults to False.
+    If set to True it enables you to decode objects serialized in jsonpickle v1.
+    Please do not attempt to re-encode the objects in the v1 format! Version 2's
+    format fixes issue #255, and allows dictionary identity to be preserve through
+    an encode/decode cycle.
+
     >>> decode('"my string"') == 'my string'
     True
     >>> decode('36')
     36
     """
     backend = backend or json
-    context = context or Unpickler(keys=keys, backend=backend, safe=safe)
+    context = context or Unpickler(
+        keys=keys, backend=backend, safe=safe, v1_decode=v1_decode
+    )
     data = backend.decode(string)
     return context.restore(data, reset=reset, classes=classes)
 
@@ -50,6 +74,11 @@ def _safe_hasattr(obj, attr):
         return False
 
 
+def _is_json_key(key):
+    """Has this key a special object that has been encoded to JSON?"""
+    return isinstance(key, compat.string_types) and key.startswith(tags.JSON_KEY)
+
+
 class _Proxy(object):
     """Proxies are dummy objects that are later replaced by real instances
 
@@ -78,6 +107,7 @@ class _Proxy(object):
     when swapping proxies with real instances.
 
     """
+
     def __init__(self):
         self.instance = None
 
@@ -106,17 +136,16 @@ def _obj_setvalue(obj, idx, proxy):
 
 
 class Unpickler(object):
-
-    def __init__(self, backend=None, keys=False, safe=False):
+    def __init__(self, backend=None, keys=False, safe=False, v1_decode=False):
         self.backend = backend or json
         self.keys = keys
         self.safe = safe
+        self.v1_decode = v1_decode
 
         self.reset()
 
     def reset(self):
-        """Resets the object's internal state.
-        """
+        """Resets the object's internal state."""
         # Map reference names to object instances
         self._namedict = {}
 
@@ -171,41 +200,65 @@ class Unpickler(object):
         self._proxies = []
 
     def _restore(self, obj):
-        if has_tag(obj, tags.B64):
-            restore = self._restore_base64
-        elif has_tag(obj, tags.B85):
-            restore = self._restore_base85
-        elif has_tag(obj, tags.BYTES):  # Backwards compatibility
-            restore = self._restore_quopri
-        elif has_tag(obj, tags.ID):
-            restore = self._restore_id
-        elif has_tag(obj, tags.REF):  # Backwards compatibility
-            restore = self._restore_ref
-        elif has_tag(obj, tags.ITERATOR):
-            restore = self._restore_iterator
-        elif has_tag(obj, tags.TYPE):
-            restore = self._restore_type
-        elif has_tag(obj, tags.REPR):  # Backwards compatibility
-            restore = self._restore_repr
-        elif has_tag(obj, tags.REDUCE):
-            restore = self._restore_reduce
-        elif has_tag(obj, tags.OBJECT):
-            restore = self._restore_object
-        elif has_tag(obj, tags.FUNCTION):
-            restore = self._restore_function
-        elif util.is_list(obj):
-            restore = self._restore_list
-        elif has_tag(obj, tags.TUPLE):
-            restore = self._restore_tuple
-        elif has_tag(obj, tags.SET):
-            restore = self._restore_set
-        elif util.is_dictionary(obj):
-            restore = self._restore_dict
-        else:
+        # if obj isn't in these types, neither it nor nothing in it can have a tag
+        # don't change the tuple of types to a set, it won't work with isinstance
+        if not isinstance(obj, (str, list, dict, set, tuple)):
+
             def restore(x):
                 return x
+
+        else:
+            restore = self._restore_tags(obj)
         return restore(obj)
 
+    def _restore_tags(self, obj):
+        try:
+            if not tags.RESERVED <= set(obj) and not type(obj) in (list, dict):
+
+                def restore(x):
+                    return x
+
+                return restore
+        except TypeError:
+            pass
+        if type(obj) is dict:
+            if has_tag_dict(obj, tags.TUPLE):
+                restore = self._restore_tuple
+            elif has_tag_dict(obj, tags.SET):
+                restore = self._restore_set
+            elif has_tag_dict(obj, tags.B64):
+                restore = self._restore_base64
+            elif has_tag_dict(obj, tags.B85):
+                restore = self._restore_base85
+            elif has_tag_dict(obj, tags.ID):
+                restore = self._restore_id
+            elif has_tag_dict(obj, tags.ITERATOR):
+                restore = self._restore_iterator
+            elif has_tag_dict(obj, tags.OBJECT):
+                restore = self._restore_object
+            elif has_tag_dict(obj, tags.TYPE):
+                restore = self._restore_type
+            elif has_tag_dict(obj, tags.REDUCE):
+                restore = self._restore_reduce
+            elif has_tag_dict(obj, tags.FUNCTION):
+                restore = self._restore_function
+            elif has_tag_dict(obj, tags.BYTES):  # Backwards compatibility
+                restore = self._restore_quopri
+            elif has_tag_dict(obj, tags.REF):  # Backwards compatibility
+                restore = self._restore_ref
+            elif has_tag_dict(obj, tags.REPR):  # Backwards compatibility
+                restore = self._restore_repr
+            else:
+                restore = self._restore_dict
+        elif util.is_list(obj):
+            restore = self._restore_list
+        else:
+
+            def restore(x):
+                return x
+
+        return restore
+
     def _restore_base64(self, obj):
         return util.b64decode(obj[tags.B64].encode('utf-8'))
 
@@ -353,8 +406,7 @@ class Unpickler(object):
         if kwargs:
             kwargs = self._restore(kwargs)
 
-        is_oldstyle = (
-            not (isinstance(cls, type) or getattr(cls, '__meta__', None)))
+        is_oldstyle = not (isinstance(cls, type) or getattr(cls, '__meta__', None))
         try:
             if (not is_oldstyle) and hasattr(cls, '__new__'):
                 # new style classes
@@ -383,18 +435,21 @@ class Unpickler(object):
         if isinstance(instance, tuple):
             return instance
 
-        if (_safe_hasattr(instance, 'default_factory') and
-                isinstance(instance.default_factory, _Proxy)):
+        instance = self._restore_object_instance_variables(obj, instance)
+
+        if _safe_hasattr(instance, 'default_factory') and isinstance(
+            instance.default_factory, _Proxy
+        ):
             instance.default_factory = instance.default_factory.get()
 
-        return self._restore_object_instance_variables(obj, instance)
+        return instance
 
     def _restore_from_dict(self, obj, instance, ignorereserved=True):
         restore_key = self._restore_key_fn()
         method = _obj_setattr
         deferred = {}
 
-        for k, v in sorted(obj.items(), key=util.itemgetter):
+        for k, v in util.items(obj):
             # ignore the reserved attribute
             if ignorereserved and k in tags.RESERVED:
                 continue
@@ -406,8 +461,7 @@ class Unpickler(object):
             k = restore_key(k)
             # step into the namespace
             value = self._restore(v)
-            if (util.is_noncomplex(instance) or
-                    util.is_dictionary_subclass(instance)):
+            if util.is_noncomplex(instance) or util.is_dictionary_subclass(instance):
                 try:
                     if k == '__dict__':
                         setattr(instance, k, value)
@@ -444,7 +498,7 @@ class Unpickler(object):
             if hasattr(instance, 'append'):
                 for v in obj[tags.SEQ]:
                     instance.append(self._restore(v))
-            if hasattr(instance, 'add'):
+            elif hasattr(instance, 'add'):
                 for v in obj[tags.SEQ]:
                     instance.add(self._restore(v))
 
@@ -455,8 +509,9 @@ class Unpickler(object):
 
     def _restore_state(self, obj, instance):
         state = self._restore(obj[tags.STATE])
-        has_slots = (isinstance(state, tuple) and len(state) == 2
-                     and isinstance(state[1], dict))
+        has_slots = (
+            isinstance(state, tuple) and len(state) == 2 and isinstance(state[1], dict)
+        )
         has_slots_and_dict = has_slots and isinstance(state[0], dict)
         if hasattr(instance, '__setstate__'):
             instance.__setstate__(state)
@@ -464,16 +519,16 @@ class Unpickler(object):
             # implements described default handling
             # of state for object with instance dict
             # and no slots
-            instance = self._restore_from_dict(
-                    state, instance, ignorereserved=False)
+            instance = self._restore_from_dict(state, instance, ignorereserved=False)
         elif has_slots:
-            instance = self._restore_from_dict(
-                    state[1], instance, ignorereserved=False)
+            instance = self._restore_from_dict(state[1], instance, ignorereserved=False)
             if has_slots_and_dict:
                 instance = self._restore_from_dict(
-                        state[0], instance, ignorereserved=False)
-        elif (not hasattr(instance, '__getnewargs__')
-              and not hasattr(instance, '__getnewargs_ex__')):
+                    state[0], instance, ignorereserved=False
+                )
+        elif not hasattr(instance, '__getnewargs__') and not hasattr(
+            instance, '__getnewargs_ex__'
+        ):
             # __setstate__ is not implemented so that means that the best
             # we can do is return the result of __getstate__() rather than
             # return an empty shell of an object.
@@ -503,20 +558,50 @@ class Unpickler(object):
 
     def _restore_dict(self, obj):
         data = {}
-        restore_key = self._restore_key_fn()
-        for k, v in sorted(obj.items(), key=util.itemgetter):
-            if isinstance(k, numeric_types):
-                str_k = k.__str__()
-            else:
-                str_k = k
-            self._namestack.append(str_k)
-            k = restore_key(k)
-            data[k] = self._restore(v)
-            # k is currently a proxy and must be replaced
-            if isinstance(data[k], _Proxy):
-                self._proxies.append((data, k, data[k], _obj_setvalue))
+        if not self.v1_decode:
+            self._mkref(data)
 
-            self._namestack.pop()
+        # If we are decoding dicts that can have non-string keys then we
+        # need to do a two-phase decode where the non-string keys are
+        # processed last.  This ensures a deterministic order when
+        # assigning object IDs for references.
+        if self.keys:
+            # Phase 1: regular non-special keys.
+            for k, v in util.items(obj):
+                if _is_json_key(k):
+                    continue
+                if isinstance(k, numeric_types):
+                    str_k = k.__str__()
+                else:
+                    str_k = k
+                self._namestack.append(str_k)
+                data[k] = self._restore(v)
+
+                self._namestack.pop()
+
+            # Phase 2: object keys only.
+            for k, v in util.items(obj):
+                if not _is_json_key(k):
+                    continue
+                self._namestack.append(k)
+
+                k = self._restore_pickled_key(k)
+                data[k] = result = self._restore(v)
+                # k is currently a proxy and must be replaced
+                if isinstance(result, _Proxy):
+                    self._proxies.append((data, k, result, _obj_setvalue))
+
+                self._namestack.pop()
+        else:
+            # No special keys, thus we don't need to restore the keys either.
+            for k, v in util.items(obj):
+                if isinstance(k, numeric_types):
+                    str_k = k.__str__()
+                else:
+                    str_k = k
+                self._namestack.append(str_k)
+                data[k] = self._restore(v)
+                self._namestack.pop()
         return data
 
     def _restore_key_fn(self):
@@ -534,17 +619,22 @@ class Unpickler(object):
         if self.keys:
             restore_key = self._restore_pickled_key
         else:
+
             def restore_key(key):
                 return key
+
         return restore_key
 
     def _restore_pickled_key(self, key):
         """Restore a possibly pickled key"""
-        if (isinstance(key, compat.string_types) and
-                key.startswith(tags.JSON_KEY)):
-            key = decode(key[len(tags.JSON_KEY):],
-                         backend=self.backend, context=self,
-                         keys=True, reset=False)
+        if _is_json_key(key):
+            key = decode(
+                key[len(tags.JSON_KEY) :],
+                backend=self.backend,
+                context=self,
+                keys=True,
+                reset=False,
+            )
         return key
 
     def _refname(self):
@@ -619,7 +709,7 @@ def loadclass(module_and_name, classes=None):
     # First assume that everything up to the last dot is the module name,
     # then try other splits to handle classes that are defined within
     # classes
-    for up_to in range(len(names)-1, 0, -1):
+    for up_to in range(len(names) - 1, 0, -1):
         module = util.untranslate_module_name('.'.join(names[:up_to]))
         try:
             __import__(module)
@@ -627,7 +717,7 @@ def loadclass(module_and_name, classes=None):
             for class_name in names[up_to:]:
                 obj = getattr(obj, class_name)
             return obj
-        except (AttributeError, ImportError, ValueError) as ex:
+        except (AttributeError, ImportError, ValueError):
             continue
     return None
 
@@ -708,3 +798,20 @@ def has_tag(obj, tag):
 
     """
     return type(obj) is dict and tag in obj
+
+
+def has_tag_dict(obj, tag):
+    """Helper class that tests to see if the obj is a dictionary
+    and contains a particular key/tag.
+
+    >>> obj = {'test': 1}
+    >>> has_tag(obj, 'test')
+    True
+    >>> has_tag(obj, 'fail')
+    False
+
+    >>> has_tag(42, 'fail')
+    False
+
+    """
+    return tag in obj
