@@ -4,7 +4,6 @@ import unittest
 from warnings import warn
 
 import jsonpickle
-from jsonpickle import compat
 from jsonpickle.compat import PY2
 from jsonpickle.compat import PY3
 
@@ -12,7 +11,6 @@ from helper import SkippableTest
 
 
 class Thing(object):
-
     def __init__(self, name):
         self.name = name
         self.child = None
@@ -22,7 +20,6 @@ SAMPLE_DATA = {'things': [Thing('data')]}
 
 
 class BackendBase(SkippableTest):
-
     def _is_installed(self, backend):
         if not jsonpickle.util.is_installed(backend):
             return self.skip('%s not available; please install' % backend)
@@ -62,6 +59,15 @@ class BackendBase(SkippableTest):
         actual = jsonpickle.decode(pickle)
         self.assertEqual(expect, actual)
 
+    def test_encode_with_indent_and_separators(self):
+        obj = {
+            'a': 1,
+            'b': 2,
+        }
+        expect = '{\n' '    "a": 1,\n' '    "b": 2\n' '}'
+        actual = jsonpickle.encode(obj, indent=4, separators=(',', ': '))
+        self.assertEqual(expect, actual)
+
 
 class JsonTestCase(BackendBase):
     def setUp(self):
@@ -73,7 +79,8 @@ class JsonTestCase(BackendBase):
             '"py/object": "backend_test.Thing", '
             '"name": "data", '
             '"child": null} '
-            ']}')
+            ']}'
+        )
         self.assertEncodeDecode(expected_pickled)
 
 
@@ -87,7 +94,8 @@ class SimpleJsonTestCase(BackendBase):
             '"py/object": "backend_test.Thing", '
             '"name": "data", '
             '"child": null}'
-            ']}')
+            ']}'
+        )
         self.assertEncodeDecode(expected_pickled)
 
     def test_decimal(self):
@@ -99,11 +107,9 @@ class SimpleJsonTestCase(BackendBase):
         self.assertEqual(obj, clone)
 
         # Custom behavior: we want to use simplejson's Decimal support.
-        jsonpickle.set_encoder_options('simplejson',
-                                       use_decimal=True, sort_keys=True)
+        jsonpickle.set_encoder_options('simplejson', use_decimal=True, sort_keys=True)
 
-        jsonpickle.set_decoder_options('simplejson',
-                                       use_decimal=True)
+        jsonpickle.set_decoder_options('simplejson', use_decimal=True)
 
         # use_decimal mode allows Decimal objects to pass-through to simplejson.
         # The end result is we get a simple '0.5' value as our json string.
@@ -124,34 +130,11 @@ def has_module(module):
     try:
         __import__(module)
     except ImportError:
-        warn(module + ' module not available for testing, '
-             'consider installing')
+        warn(module + ' module not available for testing, ' 'consider installing')
         return False
     return True
 
 
-class DemjsonTestCase(BackendBase):
-
-    def setUp(self):
-        self.set_preferred_backend('demjson')
-
-    def test_backend(self):
-        expected_pickled = compat.ustr(
-            '{"things":[{'
-            '"child":null,'
-            '"name":"data",'
-            '"py/object":"backend_test.Thing"}'
-            ']}')
-        self.assertEncodeDecode(expected_pickled)
-
-    def test_int_dict_keys_with_numeric_keys(self):
-        jsonpickle.set_encoder_options('demjson', strict=False)
-        int_dict = {1000: [1, 2]}
-        pickle = jsonpickle.encode(int_dict, numeric_keys=True)
-        actual = jsonpickle.decode(pickle)
-        self.assertEqual(actual[1000], [1, 2])
-
-
 class JsonlibTestCase(BackendBase):
     def setUp(self):
         if PY2:
@@ -164,7 +147,8 @@ class JsonlibTestCase(BackendBase):
             '{"things":[{'
             r'"py\/object":"backend_test.Thing",'
             '"name":"data","child":null}'
-            ']}')
+            ']}'
+        )
         self.assertEncodeDecode(expected_pickled)
 
 
@@ -177,12 +161,12 @@ class YajlTestCase(BackendBase):
             '{"things":[{'
             '"py/object":"backend_test.Thing",'
             '"name":"data","child":null}'
-            ']}')
+            ']}'
+        )
         self.assertEncodeDecode(expected_pickled)
 
 
 class UJsonTestCase(BackendBase):
-
     def setUp(self):
         self.set_preferred_backend('ujson')
 
@@ -191,7 +175,8 @@ class UJsonTestCase(BackendBase):
             '{"things":[{'
             r'"py\/object":"backend_test.Thing",'
             '"name":"data","child":null}'
-            ']}')
+            ']}'
+        )
         self.assertEncodeDecode(expected_pickled)
 
 
@@ -200,8 +185,6 @@ def suite():
     suite.addTest(unittest.makeSuite(JsonTestCase))
     suite.addTest(unittest.makeSuite(UJsonTestCase))
     suite.addTest(unittest.makeSuite(SimpleJsonTestCase))
-    if has_module('demjson'):
-        suite.addTest(unittest.makeSuite(DemjsonTestCase))
     if has_module('yajl'):
         suite.addTest(unittest.makeSuite(YajlTestCase))
     if PY2:
