@@ -1,22 +1,21 @@
 # Copyright (C) 2008 John Paulett (john -at- paulett.org)
-# Copyright (C) 2009-2018 David Aguilar (davvid -at- gmail.com)
+# Copyright (C) 2009-2021 David Aguilar (davvid -at- gmail.com)
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.
 from __future__ import absolute_import, division, unicode_literals
-import doctest
 import os
 import unittest
 import collections
 
+import pytest
+
 import jsonpickle
 import jsonpickle.backend
 import jsonpickle.handlers
-
 from jsonpickle import compat, tags, util
 from jsonpickle.compat import PY2, PY3
-
 from helper import SkippableTest
 
 
@@ -73,7 +72,7 @@ class ThingWithProps(object):
         self.name = state_dict.get('nom', '')
         ident = state_dict.get('__identity__')
         if ident != self.identity:
-            raise ValueError('expanded object does not match originial state!')
+            raise ValueError('expanded object does not match original state!')
 
     def __eq__(self, other):
         return self.identity == other.identity
@@ -685,6 +684,7 @@ class JSONPickleTestCase(SkippableTest):
             and 'os.path' in jsonpickle.json._encoder_options
             and 'os.path' in jsonpickle.json._decoder_exceptions
         )
+        jsonpickle.remove_backend('os.path')
 
     def _backend_is_partially_loaded(self, backend):
         """Return True if the specified backend is incomplete"""
@@ -831,6 +831,10 @@ class JSONPickleTestCase(SkippableTest):
         self.assertEqual(decoded_middle_obj.attribute, middle_obj.attribute)
         self.assertEqual(decoded_inner_obj.attribute, inner_obj.attribute)
 
+    def test_v1_decode(self):
+        # TODO: Find a simple example that reproduces #364
+        self.assertTrue(True)
+
 
 class PicklableNamedTuple(object):
     """
@@ -1133,7 +1137,7 @@ class PicklingProtocol4TestCase(unittest.TestCase):
     def test_pickle_newargs_ex(self):
         """
         Ensure we can pickle and unpickle an object whose class needs arguments
-        to __new__ and get back the same typle
+        to __new__ and get back the same type
         """
         instance = PicklableNamedTupleEx(**{'a': 'b', 'n': 2})
         encoded = jsonpickle.encode(instance)
@@ -1349,7 +1353,7 @@ class PicklingProtocol2TestCase(SkippableTest):
     def test_pickle_newargs(self):
         """
         Ensure we can pickle and unpickle an object whose class needs arguments
-        to __new__ and get back the same typle
+        to __new__ and get back the same type
         """
         instance = PicklableNamedTuple(('a', 'b'), (1, 2))
         encoded = jsonpickle.encode(instance)
@@ -1530,6 +1534,15 @@ class PicklingProtocol2TestCase(SkippableTest):
         self.test_cyclical_objects_unpickleable_false(use_tuple=False)
 
 
+def test_dict_references_are_preserved():
+    data = {}
+    actual = jsonpickle.decode(jsonpickle.encode([data, data]))
+    assert isinstance(actual, list)
+    assert isinstance(actual[0], dict)
+    assert isinstance(actual[1], dict)
+    assert actual[0] is actual[1]
+
+
 def test_repeat_objects_are_expanded():
     """Ensure that all objects are present in the json output"""
     # When references are disabled we should create expanded copies
@@ -1555,17 +1568,5 @@ def test_repeat_objects_are_expanded():
     assert flattened['passengers'][0]['child']['name'] == 'bob'
 
 
-def suite():
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(JSONPickleTestCase))
-    suite.addTest(unittest.makeSuite(PicklingTestCase))
-    suite.addTest(unittest.makeSuite(PicklingProtocol2TestCase))
-    suite.addTest(unittest.makeSuite(PicklingProtocol4TestCase))
-    suite.addTest(doctest.DocTestSuite(jsonpickle))
-    suite.addTest(doctest.DocTestSuite(jsonpickle.pickler))
-    suite.addTest(doctest.DocTestSuite(jsonpickle.unpickler))
-    return suite
-
-
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    pytest.main([__file__])
